import torch
import torch.nn as nn
from numpy.random import normal
from numpy.linalg import svd
from math import sqrt
import torch.nn.init
from .common import *

class ResidualSequential(nn.Sequential):
    def __init__(self, *args):
        super(ResidualSequential, self).__init__(*args)

    def forward(self, x):
        out = super(ResidualSequential, self).forward(x)
        x_ = None
        if out.size(2) != x.size(2) or out.size(3) != x.size(3):
            diff2 = x.size(2) - out.size(2)
            diff3 = x.size(3) - out.size(3)
            x_ = x[:, :, diff2 /2:out.size(2) + diff2 / 2, diff3 / 2:out.size(3) + diff3 / 2]
        else:
            x_ = x
        return out + x_

    def eval(self):
        for m in self.modules():
            m.eval()
        exit()


def get_block(num_channels, norm_layer, act_fun):
    layers = [
        nn.Conv2d(num_channels, num_channels, 3, 1, 1, bias=False),
        norm_layer(num_channels, affine=True),
        act(act_fun),
        nn.Conv2d(num_channels, num_channels, 3, 1, 1, bias=False),
        norm_layer(num_channels, affine=True),
    ]
    return layers


class ResNet(nn.Module):
    def __init__(self, num_input_channels,
                 num_output_channels,
                 num_blocks,
                 num_channels,
                 need_residual = True,
                 act_fun = 'LeakyReLU',
                 need_sigmoid = True,
                 norm_layer = nn.BatchNorm2d,
                 pad = 'reflection'):
        super(ResNet, self).__init__()
        if need_residual:
            s = ResidualSequential
        else:
            s = nn.Sequential
        stride = 1
        layers = [conv(in_f = num_input_channels, out_f = num_channels, kernel_size = 3, stride = stride, bias = True, pad = pad, downsample_mode = 'stride'), act(act_fun)]
        # Residual blocks
        for i in range(num_blocks):
            layers += [s(*get_block(num_channels, norm_layer, act_fun)), act(act_fun)]
        # output layer
        layers += [conv(in_f = num_channels, out_f = num_channels, kernel_size = 3, stride = stride, pad = pad), norm_layer(num_channels, affine = True), conv(num_channels, num_output_channels, kernel_size = 3, stride = stride, bias = True, pad = pad)]
        if need_sigmoid:
            layers += [nn.Sigmoid()]
        self.model = nn.Sequential(*layers)

    def forward(self, input):
        return self.model(input)

    def eval(self):
        self.model.eval()
